import 'package:flutter/material.dart';

//加载网页库
import 'package:webview_flutter/webview_flutter.dart';
import 'dart:io';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      //去掉debug标签
      debugShowCheckedModeBanner: false,
      title: 'Flutter Demo',
      home: Test(),
    );
  }
}

class Test extends StatefulWidget {
  Test({Key key}) : super(key: key);

  @override
  _TestState createState() => _TestState();
}

class _TestState extends State<Test> {
  //创建webview控制器 用于控制页面的返回
  WebViewController _controller;

  @override
  void initState() {
    super.initState();
    //初始化 必须的
    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
  }

  @override
  Widget build(BuildContext context) {
    //WillPopScope 可以监听导航键 比如返回按钮
    return WillPopScope(
      //导航键回调方法
      onWillPop: _onWillPop,
      child: Scaffold(
        //去掉顶部栏 并预留出状态栏的高度
        appBar: PreferredSize(
          preferredSize:Size.fromHeight(MediaQuery.of(context).size.height * 0.07),
          child:SafeArea(
            top: true,
            child: Offstage(),
          ),
        ),
        //内容页面 webview
        body: WebView(
          //默认打开的网址
          initialUrl: 'https://www.bilibili.com/',
          //是否使用js
          javascriptMode: JavascriptMode.unrestricted,
          //页面创建完成以后回调函数
          onWebViewCreated: (WebViewController controller) {
            //给控制器赋值 不这样做返回页面不起作用
            _controller = controller;
          },
        ),
      ),
    );
  }

  //返回回调方法 返回的布尔值 true就是确定按了返回键 false就是没有按返回键
  Future<bool> _onWillPop() async {
    //获取有没有历史页面
    var status = await _controller.canGoBack();
    //如果返回true 也就是有历史页面 可以返回
    if (status) {
      //返回之前的页面
      _controller.goBack();
      //返回false就是没有按返回键
      return false;
    } else {
      //如果没有历史页面 说明在主页
      //显示退出提示框 询问用户是否退出
      return (await showDialog(
        context: context,
        builder: (context) => AlertDialog(
          title: Text('确定要退出吗?'),
          content: Text('点击确定将退出应用程序'),
          actions: [
            TextButton(
              onPressed: () => Navigator.of(context).pop(false),
              child: Text('取消'),
            ),
            TextButton(
              onPressed: () => Navigator.of(context).pop(true),
              child: Text('退出'),
            ),
          ],
        ),
      )) ?? false;

    }
  }

}

