package main

import (
	"fastapp/routers"
	"fastapp/utils/logs"
	"fastapp/utils/task"
	"fmt"
	"html/template"
	"io"
	"net/http"
	"os"
	"time"

	"embed"

	"github.com/Unknwon/goconfig"
	"github.com/gin-gonic/gin"
)

//go:embed assets/*
var assets embed.FS

//go:embed assets/favicon.ico
var favicon embed.FS

//go:embed view/*
var view embed.FS

func main() {
	// 禁用控制台颜色
	gin.DisableConsoleColor()

	// 创建记录日志的文件
	f, _ := os.Create("gin.log")
	gin.DefaultWriter = io.MultiWriter(f)

	// 创建一个默认的路由引擎
	r := gin.Default() //带log等中间件
	// r := gin.New()//不带中间件的路由引擎

	//加载静态文件
	r.StaticFS("/public", http.FS(assets)) //自定义文件服务器 http://127.0.0.1/public/assets/layui/layui.js
	//加载单个图标静态文件 http://127.0.0.1/favicon.ico
	r.GET("favicon.ico", func(c *gin.Context) {
		//其实不用单独一个嵌入 可以使用assets
		file, _ := favicon.ReadFile("assets/favicon.ico")
		// file, _ := assets.ReadFile("assets/favicon.ico")
		c.Data(
			http.StatusOK,
			"image/x-icon",
			file,
		)
	})

	//自定义函数
	var funcMap = template.FuncMap{
		"showtime":  Showtime,
		"add":       Add,
		"unescaped": unescaped,
	}

	//加载模板文件
	//加载自定义函数
	templ := template.Must(template.New("").Funcs(funcMap).ParseFS(view, "view/*.html"))
	r.SetHTMLTemplate(templ)

	//注册路由
	routers.IndexRouter(r) //主页路由
	routers.AdminRouter(r) //后台路由

	//启动一个协程 调用执行生成app任务的方法
	go task.Task()

	// 启动HTTP服务，默认在127.0.0.1:8080启动服务
	r.Run(":80") //也可以写全乎了比如"127.0.0.1:80"或者域名
}

func test() {
	// 加载配置文件
	cfg, err := goconfig.LoadConfigFile("conf.ini")
	if err != nil {
		logs.Info("加载配置文件出错", err)
	}

	//读取配置项
	system := cfg.MustValue("", "system")
	fmt.Println(system)
}

//相加 自定义函数
func Add(a, b int) int {
	return a + b
}

//转换格式化时间戳 自定义函数
func Showtime(time1 int) string {
	t := time.Unix(int64(time1), 0)
	// fmt.Println(time1) //输出 2019-07-08 11:17:39 +0800 CST
	//转换完以后就可以格式化输出
	return t.Format("2006-01-02 15:04:05")
	// fmt.Println() //输出 2019-07-08 11:17:39
}

//转换html 自定义函数
func unescaped(x string) interface{} {
	return template.HTML(x)
}
