package routers

import (
	"fastapp/model"
	"fastapp/utils/logs"
	"fastapp/utils/utils"
	"io/ioutil"
	"net/http"
	"path"
	"strconv"
	"time"

	"github.com/gin-gonic/gin"
)

const (
	b  = 1
	kb = b * 1024
	mb = b * 1024 * 1024
	gb = b * 1024 * 1024 * 1024
	tb = b * 1024 * 1024 * 1024 * 1042
	pb = b * 1024 * 1024 * 1024 * 1042 * 1024
	eb = b * 1024 * 1024 * 1024 * 1042 * 1024 * 1024
)
const (
	B  = 1
	KB = b * 1000
	MB = b * 1000 * 1000
	GB = b * 1000 * 1000 * 1000
	TB = b * 1000 * 1000 * 1000 * 1000
	PB = b * 1000 * 1000 * 1000 * 1000 * 1000
	EB = b * 1000 * 1000 * 1000 * 1000 * 1000 * 1000
)

//IndexRouter 主页路由
func IndexRouter(e *gin.Engine) {
	e.GET("/", index)
	e.POST("/addapp", addapp)
	e.GET("/search", search)
	e.GET("/down", down)
}

//主页
func index(c *gin.Context) {
	//sql语句
	查询后台数据 := "select * from admin"
	var 后台数据 model.Admin
	//执行sql
	utils.Db.Get(&后台数据, 查询后台数据)
	c.HTML(http.StatusOK, "index.html", 后台数据)
}

//添加制作app
func addapp(c *gin.Context) {
	//获取表单参数
	url := c.PostForm("url")
	// fmt.Println(url)
	//如果网址为空 返回错误
	if url == "" {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "网址错误",
		})
		return
	}

	//获取表单参数
	search := c.PostForm("search")
	// fmt.Println(url)
	//如果查询密码为空 返回错误
	if search == "" {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "未输入查询密码",
		})
		return
	}

	//获取上传文件
	file, err := c.FormFile("file")
	//如果上传文件为空 返回错误
	if err != nil {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "应用图片未添加",
		})
		return
	}

	// fmt.Println(file.Filename) //打印一下文件名
	// fmt.Println(file.Size)     //打印一下文件大小

	//如果上传文件超过1mb
	if file.Size > mb {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "应用图片超过1mb",
		})
		return
	}

	//返回路径中的扩展名
	//如果没有点，返回空
	extstring := path.Ext(file.Filename)

	//允许上传文件的格式
	allowExtMap := map[string]bool{
		".jpg":  true,
		".png":  true,
		".gif":  true,
		".jpeg": true,
	}
	//打印 如果map里有这个格式 返回true 没有则返回false
	// println(allowExtMap[extstring])

	//取反 也就是有这个格式返回false 没有返回true
	if !allowExtMap[extstring] {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "应用图片格式错误",
		})
		return
	}

	//查询排队进度
	查询排队进度sql语句 := "select count(*) from task where status=?"
	//排队变量
	var count int
	//执行sql
	err = utils.Db.Get(&count, 查询排队进度sql语句, 0)
	if err != nil {
		count = 0
	}
	//如果排队人数大于99人
	if count > 98 {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "当前排队人数超过100人,暂停添加,请稍后在进行添加.",
		})
		return
	}

	//生成新文件名及路径
	filename := "./image/" + strconv.FormatInt(time.Now().Unix(), 10) + extstring
	// println(filename)
	//保存文件 参数(文件,路径/文件名) 只使用文件名默认是本文件夹
	err = c.SaveUploadedFile(file, filename)
	//如果保存失败
	if err != nil {
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "应用图片上传错误",
		})
		return
	}

	//添加应用sql语句
	sqlStr := "insert into task(addtime, url ,search ,image , status ,download,endtime) values (?,?,?,?,?,?,?)"
	//执行添加应用sql语句
	_, err = utils.Db.Exec(sqlStr, time.Now().Unix(), url, search, filename, 0, "", "")
	if err != nil {
		logs.Debug("生成app入库时发生错误", err)
		c.JSON(200, gin.H{
			"code": "1",
			"msg":  "添加至生成队列失败",
		})
		return
	}

	c.JSON(200, gin.H{
		"code": "0",
		"msg":  "添加成功，排队中",
	})
}

//制作app进度查询
func search(c *gin.Context) {
	//获取get参数
	url := c.Query("url")
	search := c.Query("search")
	// fmt.Println(url, "|", search)

	//sql语句
	查询后台数据 := "select * from admin"
	var 后台数据 model.Admin
	//执行sql
	utils.Db.Get(&后台数据, 查询后台数据)

	//如果没有参数
	if url == "" || search == "" {
		var date model.Index
		date.Admin = 后台数据
		c.HTML(http.StatusOK, "search.html", date)
		return
	}

	//通过查询参数 获取任务详情
	task, err := getid(url, search)
	//如果出现错误 说明没有或者参数输入错误
	if err != nil {
		var date model.Index
		date.Msg = "查询参数错误"
		c.HTML(http.StatusOK, "search.html", date)
		return
	}

	rank := getrank(task.Id)
	time := gettime(task.Addtime, rank)
	var date model.Index
	date.Msg = "查询成功"
	date.Task = task
	date.Rank = rank
	date.Endtime = time
	date.Admin = 后台数据
	c.HTML(http.StatusOK, "search.html", date)

}

//获取任务id
func getid(url string, search string) (model.Task, error) {
	//sql语句
	sqlStr := "select * from task where url=? and search=?"
	//声明model 下的结构体
	var task model.Task
	//执行sql
	err := utils.Db.Get(&task, sqlStr, url, search)
	if err != nil {
		return task, err
	}
	return task, nil
}

//获取排队进度
func getrank(id int) int {
	//sql语句
	sqlStr := "select count(*) from task where id<? and status=?"
	//声明model 下的结构体
	var count int
	//执行sql
	err := utils.Db.Get(&count, sqlStr, id, 0)
	if err != nil {
		count = 0
	}
	return count
}

//获取预计可下载时间
func gettime(addtime, rank int) string {
	//给队列+1 要不0的话不增加时间
	rank = rank + 1
	//时间往后延迟300秒
	l := addtime + rank*300
	t := time.Unix(int64(l), 0)
	// fmt.Println(time1) //输出 2019-07-08 11:17:39 +0800 CST
	//转换完以后就可以格式化输出
	return t.Format("2006-01-02 15:04:05")
	// fmt.Println() //输出 2019-07-08 11:17:39
}

//down 下载apk方法
func down(c *gin.Context) {
	//获取get参数
	down := c.Query("down")

	// fmt.Println(down)

	//如果没有参数
	if down == "" {
		// c.HTML(http.StatusOK, "search.html", nil)
		c.String(200, "参数错误")
		return
	}

	//打开文件
	file, err := ioutil.ReadFile(down)
	if err != nil {
		c.String(200, "参数错误,或者文件已被删除.")
		return
	}
	//输出文件
	c.Data(
		http.StatusOK,
		"application/vnd.android.package-archive",
		file,
	)
}
