package logs

import (
	"log"
	"os"
	"path"
	"runtime"
	"strconv"
)

//日志文件路径
var filename = "./log/日志.log"

//Info 普通日志不影响程序运行
func Info(info string, err error) {
	//获取调用方法的文件名\方法名\行号方法
	fname, funcname, lineno := getLineInfo()
	//打开文件(参数 文件路径,读写打开|追加文件|没有就创建,创建文件的权限)
	f, _ := os.OpenFile(filename, os.O_RDWR|os.O_APPEND|os.O_CREATE, 0777)
	//关闭文件
	defer f.Close()
	//创建log对象并设置(参数 日志文件,自定义信息包括发生错误的文件/方法/行号/自定义错误信息,flag记录器 日期时间)
	logger := log.New(f, "[info]\t"+fname+"/"+funcname+"/"+strconv.Itoa(lineno)+"\t"+info+"\t", log.LstdFlags)
	//打印日志
	logger.Println(err)
}

//Debug 严重日志会影响程序运行
func Debug(info string, err error) {
	//获取调用方法的文件名\方法名\行号方法
	fname, funcname, lineno := getLineInfo()
	//打开文件(参数 文件路径,读写打开|追加文件|没有就创建,创建文件的权限)
	f, _ := os.OpenFile(filename, os.O_RDWR|os.O_APPEND|os.O_CREATE, 0777)
	//关闭文件
	defer f.Close()
	//创建log对象并设置(参数 日志文件,自定义信息包括发生错误的文件/方法/行号/自定义错误信息,flag记录器 日期时间)
	logger := log.New(f, "[info]\t"+fname+"/"+funcname+"/"+strconv.Itoa(lineno)+"\t"+info+"\t", log.LstdFlags)
	//打印日志
	logger.Println(err)
}

//Warning 致命日志会导致程序退出
func Warning(info string, err error) {
	//获取调用方法的文件名\方法名\行号方法
	fname, funcname, lineno := getLineInfo()
	//打开文件(参数 文件路径,读写打开|追加文件|没有就创建,创建文件的权限)
	f, _ := os.OpenFile(filename, os.O_RDWR|os.O_APPEND|os.O_CREATE, 0777)
	//关闭文件
	defer f.Close()
	//创建log对象并设置(参数 日志文件,自定义信息包括发生错误的文件/方法/行号/自定义错误信息,flag记录器 日期时间)
	logger := log.New(f, "[info]\t"+fname+"/"+funcname+"/"+strconv.Itoa(lineno)+"\t"+info+"\t", log.LstdFlags)
	//打印日志
	logger.Println(err)
}

//getLineInfo 获取调用方法的文件名\方法名\行号方法
func getLineInfo() (fileName string, funcName string, lineNo int) {
	//runtime.Caller(0)参数 0为Caller本身所在的文件 1为调用GetLineInfo的文件 2为调用封装GetLineInfo函数的文件
	//说白了就是深度,从0开始调用了几次,就加几
	//pc为一个计数器
	pc, file, line, ok := runtime.Caller(2)
	if ok {
		// fileName = file //获取完整的文件路径
		fileName = path.Base(file) // Base函数返回路径的最后一个元素
		//返回的pc为停在汇编/机器码什么的一个具体多少行,然后用方法获取停在这块的方法名
		funcName = runtime.FuncForPC(pc).Name()
		lineNo = line
	}
	return
}
