package task

import (
	"bufio"
	"bytes"
	"fastapp/model"
	"fastapp/utils/logs"
	"fastapp/utils/utils"
	"fmt"
	"io"
	"io/ioutil"
	"os"
	"os/exec"
	"strconv"
	"strings"
	"time"

	"github.com/Unknwon/goconfig"
	"github.com/hunterhug/go_image"
)

//Task 执行生成app任务的方法
func Task() {
	//定义一个定时器 循环执行任务 time.Second * 3
	ticker := time.Tick(time.Minute * 5) //5分钟
	for range ticker {
		fmt.Println("任务开始时间", time.Now().Format("2006-01-02 15:04:05")) //任务开始时间

		//删除已超过24小时过期app
		go 删除已超过24小时过期app()

		//查询后台任务状态是否开启
		//sql语句
		查询后台数据 := "select * from admin"
		var 后台数据 model.Admin
		//执行sql
		utils.Db.Get(&后台数据, 查询后台数据)
		if 后台数据.Status == 0 {
			//如果后台任务状态为关闭 就跳过此次循环
			continue
		}

		//读取出一条需要生成app的记录
		data, err := 读取出一条需要生成app的记录()
		if err != nil {
			//如果出现错误说明没有需要生成app的记录 就跳过此次循环
			continue
		}

		//读取项目配置项
		path, system, drive, err := 读取项目配置()
		if err != nil {
			logs.Info("加载配置文件出错", err)
		}

		//删除项目图标(保存的时候会覆盖,所以没必要 不过保留代码)
		// 删除项目图标(path)

		//修改上传图标并保存至项目
		err = 修改上传图标并保存至项目(data.Image, path)
		if err != nil {
			logs.Info("修改上传图标并保存至项目出错", err)
		}

		//修改代码
		修改代码(path, data.Url)

		//删除之前生成的apk(会覆盖不用删除)
		// err = 删除之前生成的apk(path)
		// if err != nil {
		// 	logs.Info("删除之前生成的apk出错", err)
		// }

		//执行打包命令
		执行打包命令(system, drive, path)

		//生成新文件路径及时间
		新文件时间 := time.Now().Unix()
		新文件路径 := "./apk/" + strconv.FormatInt(新文件时间, 10) + ".apk"

		//将apk改名并在新路径重新保存(换平台记得修改项目路径代码)
		// 将apk改名并在新路径重新保存("D:/flutter/learndemo/build/app/outputs/apk/release/app-release.apk", 新文件路径)
		将apk改名并在新路径重新保存("/root/learndemo/build/app/outputs/apk/release/app-release.apk", 新文件路径)

		//改变数据库任务状态
		改变数据库任务状态(新文件路径, 新文件时间, data.Id)

		//删除用户上传的图片
		删除用户上传的图片(data.Image)

		fmt.Println("任务结束时间", time.Now().Format("2006-01-02 15:04:05")) //任务结束时间
	}

}

func 读取出一条需要生成app的记录() (model.Task, error) {
	//读取出一条需要生成app的记录
	sqlStr := "select * from task where status=0 limit 1"
	//数据变量
	var data model.Task
	//执行sql
	err := utils.Db.Get(&data, sqlStr)
	return data, err
}

func 读取项目配置() (string, string, string, error) {
	//定义一个返回的错误 默认为空nil
	var errs error
	errs = nil

	// 加载配置文件
	cfg, err := goconfig.LoadConfigFile("conf.ini")
	if err != nil {
		errs = err
	}
	//读取项目位置配置项
	path, err := cfg.GetValue("", "path")
	if err != nil {
		errs = err
	}
	//读取系统配置项
	system, err := cfg.GetValue("", "system")
	if err != nil {
		errs = err
	}
	//读取盘符配置项
	drive, err := cfg.GetValue("", "Drive")
	if err != nil {
		errs = err
	}

	return path, system, drive, errs
}

func 删除项目图标(path string) {
	//删除项目图标(保存的时候会覆盖,所以没必要 不过保留代码)
	// os.Remove(path + "/android/app/src/main/res/mipmap-mdpi/ic_launcher.png")
	// os.Remove(path + "/android/app/src/main/res/mipmap-hdpi/ic_launcher.png")
	// os.Remove(path + "/android/app/src/main/res/mipmap-xhdpi/ic_launcher.png")
	// os.Remove(path + "/android/app/src/main/res/mipmap-xxhdpi/ic_launcher.png")
	// os.Remove(path + "/android/app/src/main/res/mipmap-xxxhdpi/ic_launcher.png")
}

func 修改上传图标并保存至项目(原图片路径 string, 路径 string) error {
	//定义一个返回的错误 默认为空nil
	var errs error
	errs = nil

	//修改上传图标并保存至项目
	//按照宽度和高度进行等比例缩放并保存至指定位置
	err := go_image.ThumbnailF2F(原图片路径, 路径+"/android/app/src/main/res/mipmap-mdpi/ic_launcher.png", 48, 48)
	if err != nil {
		errs = err
	}
	err = go_image.ThumbnailF2F(原图片路径, 路径+"/android/app/src/main/res/mipmap-hdpi/ic_launcher.png", 72, 72)
	if err != nil {
		errs = err
	}
	err = go_image.ThumbnailF2F(原图片路径, 路径+"/android/app/src/main/res/mipmap-xhdpi/ic_launcher.png", 96, 96)
	if err != nil {
		errs = err
	}
	err = go_image.ThumbnailF2F(原图片路径, 路径+"/android/app/src/main/res/mipmap-xxhdpi/ic_launcher.png", 144, 144)
	if err != nil {
		errs = err
	}
	err = go_image.ThumbnailF2F(原图片路径, 路径+"/android/app/src/main/res/mipmap-xxxhdpi/ic_launcher.png", 192, 192)
	if err != nil {
		errs = err
	}
	return errs
}

func 修改代码(项目路径, 网址 string) {
	//原文件路径
	filepath := "./main.dart"
	//新文件路径
	fileName := 项目路径 + "/lib/main.dart"
	//打开原文件
	file, err := os.Open(filepath)
	if err != nil {
		logs.Info("修改代码时打开原文件出错", err)
	}
	//打开新文件
	file1, err := os.OpenFile(fileName, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, os.ModePerm) //打开文件 可读写 没有就创建
	if err != nil {
		logs.Info("修改代码时打开新文件出错", err)
	}

	//延迟关闭文件
	defer file.Close()
	defer file1.Close()

	//创建reader对象
	b1 := bufio.NewReader(file) //其实底层还是调用的NewReaderSize方法 默认的缓存大小是4093

	//创建writer对象
	w1 := bufio.NewWriter(file1) //其实底层还是调用的NewWriterSize方法 默认的缓存大小是4093

	//通过上方代码 可以发现要读取到完整内容 必须循环
	for {
		//ReadString() 读取数据 返回的直接是一个字符串 需要传参数在哪里停止 比如\n就是一行
		s1, err := b1.ReadString('\n')

		//打印读取到的内容
		// fmt.Println(s1)

		//判断读取内容是否需要修改的内容
		b := strings.Contains(s1, "initialUrl")
		if b {
			// fmt.Println("读取到了需要修改的内容", s1)
			//写入数据 其实是写入缓冲区了
			w1.WriteString("          initialUrl: '" + 网址 + "',\n")
			//跳过本次循环
			continue
		}

		//写入数据 其实是写入缓冲区了
		w1.WriteString(s1)

		//如果err等于io.EOF说明读取完了 后续没内容了
		if err == io.EOF {
			fmt.Println("读取完毕。。")
			//跳出循环
			break
		}

	}

	w1.Flush() //刷新缓冲区 才可以写到文件里面去
}

func 删除之前生成的apk(路径 string) error {
	err := os.Remove(路径 + "/build/app/outputs/apk/release/app-release.apk")
	return err
}

func 执行打包命令(系统, 项目盘符, 项目路径 string) {
	//如果系统是win执行
	if 系统 == "win" {
		//创建命令对象
		cmd := exec.Command("cmd")
		//定义标准输入变量
		in := bytes.NewBuffer(nil)
		//将cmd的标准输入与变量绑定
		cmd.Stdin = in //标准输入
		//写入标准输入
		in.WriteString(项目盘符 + "\n")            //写入你的命令，可以有多行，"\n"表示回车
		in.WriteString("cd " + 项目路径 + " \n")   //写入你的命令，可以有多行，"\n"表示回车
		in.WriteString("flutter build apk \n") //写入你的命令，可以有多行，"\n"表示回车
		err := cmd.Run()
		if err != nil {
			logs.Debug("win下生成apk错误", err)
		}
	} else {
		//如果系统是linux执行
		//创建命令对象
		cmd := exec.Command("sh", "./build.sh")
		cmd.Dir = "/root/" // 跳转到相应目录 linux
		err := cmd.Run()
		if err != nil {
			logs.Debug("linux下生成apk错误", err)
		}
	}
}

func 将apk改名并在新路径重新保存(项目路径, 新文件路径 string) {
	//io包读取方法
	// bs, err := ioutil.ReadFile(项目路径 + "/build/app/outputs/apk/release/app-release.apk")
	bs, err := ioutil.ReadFile(项目路径)
	if err != nil {
		logs.Info("读取生成的apk出错", err)
	}

	//io包写入方法
	err = ioutil.WriteFile(新文件路径, bs, 0777)
	if err != nil {
		logs.Info("写入生成的apk出错", err)
	}
}

func 改变数据库任务状态(新文件路径 string, 新文件时间 int64, 任务id int) {
	//sql语句
	sqlStr1 := "update task set status=? , download=? , endtime=? where id=?"
	//执行sql
	_, err := utils.Db.Exec(sqlStr1, 1, 新文件路径, 新文件时间, 任务id)
	if err != nil {
		logs.Debug("生成app后,改变数据库任务状态出错", err)
	}
}

func 删除用户上传的图片(图片路径 string) {
	err := os.Remove(图片路径)
	if err != nil {
		logs.Info("删除用户上传的图片出错", err)
	}
}

func 删除已超过24小时过期app() {
	//获取文件或目录相关信息
	fileInfoList, err := ioutil.ReadDir("./apk")
	if err != nil {
		// println("获取文件或目录相关信息错误", err)
		logs.Info("获取文件或目录相关信息错误", err)
	}

	//文件夹下文件和目录数量
	// fmt.Println(len(fileInfoList))

	//循环文件信息列表
	for i := range fileInfoList {
		//打印当前文件或目录下的文件或目录名
		// fmt.Println(fileInfoList[i].Name())
		//将文件名拆分获得时间戳
		strarr := strings.Split(fileInfoList[i].Name(), ".")
		//时间戳字符串转换为int64
		// string到int64
		int64, _ := strconv.ParseInt(strarr[0], 10, 64)
		//将时间戳转换为go时间格式
		time1 := time.Unix(int64, 0)
		//添加24小时
		h, _ := time.ParseDuration("24h") //可以正数 就是正常加 比如1小时以后
		result := time1.Add(h)
		//获得24小时后时间戳
		endtime := result.Unix()
		//获取当前时间戳
		timeunix := time.Now().Unix()

		if endtime < timeunix {
			//删除文件
			err := os.Remove("./apk/" + fileInfoList[i].Name())
			if err != nil {
				logs.Info("删除已超过24小时过期app出错", err)
			}
		}

	}
}
